#
#  Copyright 1999-2006 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

package provide MxGui 1.2

####################################################################
#                                                                  #
#     The following routine is the main routine for setup          #
#     and modification of quick scans.                             #
#                                                                  #
####################################################################

proc mxgui_quick_scan_setup { record_list scan_name scan_type \
				scan_record old_record_exists } {

# Create the toplevel widget for the scan setup screen.

	set window .quick_scan_setup

	set status [ catch { toplevel $window } ]

# If the window already exists, bring it to the top.

	if { $status != 0 } {
		raise $window
		focus $window
		return
	}

# Hide the window for now.

	wm withdraw $window
	wm title $window "Quick scan setup"

#
# Display the scan name and type at the top of the window.
#

	frame $window.name_frame
	pack $window.name_frame -padx 5 -pady 5 -expand yes -fill x

	label $window.name_frame.scan_name1 -text "Scan name:"
	pack $window.name_frame.scan_name1 -side left

	label $window.name_frame.scan_name2 -text $scan_name \
					-font $MxGui::monospaced_font
	pack $window.name_frame.scan_name2 -side left

	label $window.name_frame.pad -text " " -width 10
	pack $window.name_frame.pad -side left -expand yes -fill x

	label $window.name_frame.scan_type1 -text "Scan type:"
	pack $window.name_frame.scan_type1 -side left

	switch -- $scan_type {
		mcs_qscan { set scan_label "MCS quick scan" }
		default    { set scan_label "Unrecognized" }
	}

	label $window.name_frame.scan_type2 -text $scan_label \
					-font $MxGui::monospaced_font
	pack $window.name_frame.scan_type2 -side left

	frame $window.separator0 -height 2 -borderwidth 1 -relief sunken
	pack $window.separator0 -expand yes -fill both

#
# Initialize the display of the number of times to perform the scan.
#

	mxgui_initialize_num_scans_field $window $record_list \
				$scan_record $old_record_exists

	frame $window.separator1 -height 2 -borderwidth 1 -relief sunken
	pack $window.separator1 -expand yes -fill both

#
# Initialize the quick scan display.
#

	mxgui_quick_initialize_quick_scan_display \
			$window $record_list $scan_record \
			$old_record_exists

	frame $window.separator2 -height 2 -borderwidth 1 -relief sunken
	pack $window.separator2 -expand yes -fill both

#
# Setup the rest of the standard scan display fields.  The last argument
# to mxgui_initialize_common_scan_display is set to 1 so that the measurement
# parameters are included in the standard scan display fields.
#
	set show_measurement_parameters 1

	mxgui_initialize_standard_scan_display $window $record_list \
				$scan_record $scan_type $old_record_exists \
				$show_measurement_parameters

#
# Display the OK and Cancel buttons.
#

	frame $window.b
	pack $window.b

	set buttonbox $window.b.buttonbox

	iwidgets::buttonbox $buttonbox -padx 0 -pady 2

	$buttonbox add OK -text "Create Scan" -command \
		[ itcl::code mxgui_quick_create_scan $window $record_list \
		$scan_name $scan_record $scan_type $old_record_exists ]

	$buttonbox add Cancel -text Cancel -command \
		[ itcl::code destroy $window ]

	$buttonbox add Run -text "Create and Run Scan" -command \
	    [ itcl::code mxgui_quick_create_and_run_scan $window $record_list \
		$scan_name $scan_record $scan_type $old_record_exists ]

	$buttonbox default OK

	pack $buttonbox -padx 10 -pady 10

#
# If this is an old scan, make the contents of the form match the
# parameters of the old scan.
#

	if { $old_record_exists != 0 } {
		mxgui_quick_scan_copy_motor_fields_from_record \
				$window $record_list $scan_record

		mxgui_scan_copy_input_device_fields_from_record \
				$window $record_list $scan_record

		mxgui_scan_copy_measurement_fields_from_record \
				$window $record_list $scan_record \
				$show_measurement_parameters

		mxgui_scan_copy_datafile_and_plot_fields_from_record \
				$window $record_list $scan_record
	}

#
# Make the window appear on the screen.
#
	wm deiconify $window

	update
}

####################################################################
#                                                                  #
#  The following procedure does the initial setup of the quick     #
#  scan fields on the GUI scan setup screen.                       #
#                                                                  #
####################################################################

proc mxgui_quick_initialize_quick_scan_display { window \
				record_list scan_record old_record_exists } {
#
# Display the list of motors.
#

	set num_motors_frame $window.num_motors_frame
	set motor_frame      $window.motor_frame

	frame $num_motors_frame
	pack $num_motors_frame -padx 5 -pady 5

	set num_motors 1

	label $num_motors_frame.num_motors -text \
				"Number of motors:   1"

	pack $num_motors_frame.num_motors -side left

#
# old_num_motors is meant to be a hidden field that stores the
# old number of motors in a place the other procedures can find it.
# So don't make it visible by packing it or gridding it, etc.
#
	label $num_motors_frame.old_num_motors -text 0

# Create the frame for the list of motors.

	frame $motor_frame
	pack $motor_frame -padx 5 -pady 5

# Put labels at the top of the motor list.

	label $motor_frame.name -text "Motor name"
	label $motor_frame.start -text "Start position"
	label $motor_frame.stepsize -text "Step size"
	label $motor_frame.numsteps -text "# steps"
	label $motor_frame.end -text "End position"
	label $motor_frame.units -text "Units"
	label $motor_frame.lowlimit -text "Low limit"
	label $motor_frame.highlimit -text "High limit"

	grid $motor_frame.name $motor_frame.start $motor_frame.stepsize \
		$motor_frame.numsteps $motor_frame.end \
		$motor_frame.units $motor_frame.lowlimit \
		$motor_frame.highlimit

# Initialize the motor display.

	mxgui_quick_scan_redo_motor_fields $motor_frame $num_motors_frame \
			$record_list $scan_record $old_record_exists

}

####################################################################
#                                                                  #
#  The following routine causes the motor fields to be redisplayed #
#  after any change in the number of motors for the scan.          #
#                                                                  #
####################################################################

proc mxgui_quick_scan_redo_motor_fields { motor_frame num_motors_frame \
			record_list scan_record old_record_exists } {

	set mywidth 12

# Get the old and new number of motors.

	set old_num_motors [ $num_motors_frame.old_num_motors cget -text ]

	set num_motors 1

	if { $num_motors > $old_num_motors } {

		# Add new motors to the list of motors.

		for { set i $old_num_motors } { $i < $num_motors } { incr i } {

		    set thismotor $motor_frame.motor$i

		    iwidgets::entryfield ${thismotor}_name -width $mywidth \
			-command [ itcl::code mxgui_quick_change_motor \
				$record_list $thismotor ]

		    iwidgets::entryfield ${thismotor}_start -width $mywidth \
		-command [ itcl::code mxgui_quick_change_param scan_start \
				$record_list $thismotor ]

		    iwidgets::entryfield ${thismotor}_stepsize -width $mywidth \
		-command [ itcl::code mxgui_quick_change_param scan_stepsize \
				$record_list $thismotor ]

		    iwidgets::entryfield ${thismotor}_numsteps -width $mywidth \
		-command [ itcl::code mxgui_quick_change_param scan_numsteps \
				$record_list $thismotor ]

		    iwidgets::entryfield ${thismotor}_end -width $mywidth \
		-command [ itcl::code mxgui_quick_change_param scan_end \
				$record_list $thismotor ]

		    label ${thismotor}_units -width $mywidth

		    label ${thismotor}_lowlimit -width $mywidth

		    label ${thismotor}_highlimit -width $mywidth

		    grid ${thismotor}_name ${thismotor}_start \
			${thismotor}_stepsize ${thismotor}_numsteps \
			${thismotor}_end ${thismotor}_units \
			${thismotor}_lowlimit ${thismotor}_highlimit
		}
	} else {

		# Delete old motors from the list of motors.

		for { set i $num_motors } { $i < $old_num_motors } { incr i } {

		    set thismotor $motor_frame.motor$i

		    destroy ${thismotor}_name
		    destroy ${thismotor}_start
		    destroy ${thismotor}_stepsize
		    destroy ${thismotor}_numsteps
		    destroy ${thismotor}_end
		    destroy ${thismotor}_units
		    destroy ${thismotor}_lowlimit
		    destroy ${thismotor}_highlimit
		}
	}

	$num_motors_frame.old_num_motors configure -text $num_motors
}

####################################################################
#                                                                  #
#  The following routine causes the motor scan fields to be        #
#  initialized to the values from an old scan record.              #
#                                                                  #
####################################################################

proc mxgui_quick_scan_copy_motor_fields_from_record { window \
						record_list scan_record } {

	set num_motors    [ $scan_record get_field num_motors ]
	set motor_list    [ split [ $scan_record get_field motor_array ] ]
	set start_list    [ split [ $scan_record get_field start_position ] ]
	set end_list [ split [ $scan_record get_field end_position ] ]
	set numsteps_list [ split [ $scan_record get_field \
					requested_num_measurements ] ]

	set motor_frame $window.motor_frame

	for { set i 0 } { $i < $num_motors } { incr i } {

		set thismotor $motor_frame.motor$i

		set motor_name [ lindex $motor_list $i ]
		set start      [ lindex $start_list $i ]
		set end        [ lindex $end_list $i ]
		set numsteps   [ lindex $numsteps_list $i ]

		set stepsize [ expr ( $end - $start ) / ( $numsteps - 1 ) ]

		${thismotor}_name     insert end $motor_name
		${thismotor}_start    insert end $start
		${thismotor}_stepsize insert end $stepsize
		${thismotor}_numsteps insert end $numsteps
		${thismotor}_end      insert end $end

		set motor_record [ $record_list get_record $motor_name ]

		${thismotor}_units configure -text \
			[ $motor_record get_field units ]

		${thismotor}_lowlimit configure -text \
			[ $motor_record get_field negative_limit ]

		${thismotor}_highlimit configure -text \
			[ $motor_record get_field positive_limit ]
	}
}

####################################################################
#                                                                  #
# This procedure initializes the contents of the motor scan fields #
# when a new motor name is typed in.                               #
#                                                                  #
####################################################################

proc mxgui_quick_change_motor { record_list thismotor } {

# Get the name of the motor.

	set motor_name [ ${thismotor}_name get ]

	if [ catch { mxgui_quick_get_motor_record $record_list $motor_name } \
		motor_record ] {

		mxgui_quick_clear_motor_fields $thismotor
		return
	}

# Get the current position of the motor.

	set position [ $motor_record get_position ]

# Display the units and the limits.

	${thismotor}_units configure -text [ $motor_record get_field units ]

	${thismotor}_lowlimit configure -text \
		[ $motor_record get_field negative_limit ]

	${thismotor}_highlimit configure -text \
		[ $motor_record get_field positive_limit ]

# Set the rest of the motor fields to something innocuous.

	${thismotor}_start clear
	${thismotor}_start insert end $position

	${thismotor}_stepsize clear
	${thismotor}_stepsize insert end 0

	${thismotor}_numsteps clear
	${thismotor}_numsteps insert end 1

	${thismotor}_end clear
	${thismotor}_end insert end $position

	return
}

####################################################################
#                                                                  #
# This procedure gets the specified motor record and checks for    #
# various error conditions.                                        #
#                                                                  #
####################################################################

proc mxgui_quick_get_motor_record { record_list motor_name } {

# Does a motor by this name exist?

	if [ catch { $record_list get_record $motor_name } motor_record ] {

		Mx::error_message "No Such Motor" \
			"The motor '$motor_name' does not exist." "" ""

		return -code error ""
	}

# Is this record really a motor?

	set superclass [ $motor_record get_field mx_superclass ]
	set class [ $motor_record get_field mx_class ]

	if { $superclass != "device" } {
		# Make the error message grammatically correct.

		if { $superclass == "interface" } {
			set a_word "an"
		} else {
			set a_word "a"
		}

		set t "Record '$motor_name' is not a motor record.  "
		append t \
		"Instead it is $a_word $superclass record of class '$class'."

		Mx::error_message "Illegal Record Type" $t "" ""

		return -code error ""
	}
	if { $class != "motor" } {
		set t "Record '$motor_name' is not a motor record.  "
		append t "Instead it is a device record of class '$class'."

		Mx::error_message "Illegal Record Type" $t "" ""

		return -code error ""
	}
	return $motor_record
}

####################################################################
#                                                                  #
# This procedure checks to see if a new numerical value typed in   #
# for a motor scan field would violate the software limits for the #
# given motor.                                                     #
#                                                                  #
####################################################################

proc mxgui_quick_change_param { param_name record_list thismotor } {

	set motor_name [ ${thismotor}_name get ]

	if [ catch { mxgui_quick_get_motor_record $record_list $motor_name } \
		motor_record ] {

		mxgui_quick_clear_motor_fields $thismotor
		return
	}

	set highlimit [ $motor_record get_field positive_limit ]
	set lowlimit  [ $motor_record get_field negative_limit ]
	set units     [ $motor_record get_field units ]

	set start    [ ${thismotor}_start get ]
	set stepsize [ ${thismotor}_stepsize get ]
	set numsteps [ ${thismotor}_numsteps get ]
	set end      [ ${thismotor}_end get ]

	switch -- $param_name {
		scan_start {
			set end [ expr ($start + $stepsize * ($numsteps - 1)) ]
		}
		scan_stepsize {
			set end [ expr ($start + $stepsize * ($numsteps - 1)) ]
		}
		scan_numsteps {
			set numsteps [ expr round($numsteps) ]
			set end [ expr ($start + $stepsize * ($numsteps - 1)) ]
		}
		scan_end {
			set start [ expr ($end - $stepsize * ($numsteps - 1)) ]
		}
		default {
			log_puts "Error: param_name = '$param_name'"

			Mx::error_message "Illegal Parameter Name" \
				"Illegal Parameter Name '$param_name'" "" ""

			return -code error ""
		}
	}

# Check to see if any of the computed parameter values would exceed a limit.

	if { $stepsize >= 0 } {
		if { $start < $lowlimit } {
			set t    "Error: A start position of $start $units for "
			append t "motor '$motor_name' would exceed the lower "
			append t "limit at $lowlimit $units"

			Mx::error_message "Limit Violation" $t "" ""

			set start $lowlimit
			set end [ expr ($start + $stepsize * ($numsteps - 1)) ]
		}
		if { $end > $highlimit } {
			set t    "Error: An ending position of $end $units for "
			append t "motor '$motor_name' would exceed the upper "
			append t "limit at $highlimit $units"

			Mx::error_message "Limit Violation" $t "" ""

			if { $stepsize == 0 } {
				set numsteps 1
			} else {
				set numsteps \
				[ expr (1 + ($highlimit - $start)/$stepsize) ]
			}
			set numsteps [ expr int($numsteps) ]

			set end [ expr ($start + $stepsize * ($numsteps - 1)) ]
		}
	} else {
		if { $start > $highlimit } {
			set t    "Error: A start position of $start $units for "
			append t "motor '$motor_name' would exceed the upper "
			append t "limit at $highlimit $units"

			Mx::error_message "Limit Violation" $t "" ""

			set start $highlimit
			set end [ expr ($start + $stepsize * ($numsteps - 1)) ]
		}
		if { $end < $lowlimit } {
			set t    "Error: An ending position of $end $units for "
			append t "motor '$motor_name' would exceed the lower "
			append t "limit at $lowlimit $units"

			Mx::error_message "Limit Violation" $t "" ""

			if { $stepsize == 0 } {
				set numsteps 1
			} else {
				set numsteps \
				[ expr (1 + ($lowlimit - $start)/$stepsize) ]
			}
			set numsteps [ expr int($numsteps) ]

			set end [ expr ($start + $stepsize * ($numsteps - 1)) ]
		}
	}

# Now update the fields on the form.

	${thismotor}_start clear
	${thismotor}_start insert end $start

	${thismotor}_stepsize clear
	${thismotor}_stepsize insert end $stepsize

	${thismotor}_numsteps clear
	${thismotor}_numsteps insert end $numsteps

	${thismotor}_end clear
	${thismotor}_end insert end $end

	return
}

####################################################################
#                                                                  #
# This procedure clears all the fields for a given motor           #
#                                                                  #
####################################################################

proc mxgui_quick_clear_motor_fields { thismotor } {
	${thismotor}_name clear
	${thismotor}_start clear
	${thismotor}_stepsize clear
	${thismotor}_numsteps clear
	${thismotor}_end clear
	${thismotor}_units configure -text ""
	${thismotor}_lowlimit configure -text ""
	${thismotor}_highlimit configure -text ""
}

####################################################################
#                                                                  #
# The following routine sets up the description for the new scan.  #
#                                                                  #
####################################################################

proc mxgui_quick_create_scan { window record_list scan_name scan_record \
				scan_type old_record_exists } {

	set description "$scan_name  scan quick_scan mcs_qscan \"\" \"\" "

# Append the number of times to run this scan, the num_independent_variables
# field, and the num_motors field.

	set num_scans [ $window.num_scans_frame.num_scans get ]

	set num_independent_variables 1
	set num_motors 1

	append description "$num_scans $num_independent_variables $num_motors "

# Append the motor names, if any.

	for { set i 0 } { $i < $num_motors } { incr i } {

		set motor_name [ $window.motor_frame.motor${i}_name get ]

		set name_length [ string length [ string trim $motor_name ] ]

		if { $name_length == 0 } {
			Mx::error_message "Empty Motor Name" \
			"No motor name was specified for motor $i." "" ""

			return
		}
		append description "$motor_name "
	}

# Append the input devices, if any.

	set num_inputs_frame    $window.readout_frame.inputs.num_inputs_frame
	set input_device_frame  $window.readout_frame.inputs.input_device_frame

	set num_input_devices [ $num_inputs_frame.num_input_devices get ]

	if { $num_input_devices < 0 } {
		Mx::error_message "Illegal number of input devices" \
"The number of input devices ($num_input_devices) may not be less than zero." \
			"" ""

		return
	}
	append description "$num_input_devices "

	for { set i 0 } { $i < $num_input_devices } { incr i } {

		set input_device_name \
			[ $input_device_frame.device${i}_name get ]

		set name_length \
			[ string length [ string trim $input_device_name ] ]

		if { $name_length == 0 } {
			Mx::error_message "Empty input device name" \
		"No input device name was specified for input device $i." "" ""

			return
		}
		append description "$input_device_name "
	}

# Append the after scan action ( currently always 0 ).

	append description "0 "

# Append the settling time.

	set measurement_frame $window.readout_frame.measurement

	set settling_time [ $measurement_frame.sframe.settling_time get ]

	set length [ string length [ string trim $settling_time ] ]

	if { $length == 0 } {
		Mx::error_message "Empty settling time" \
			"No settling time was specified." "" ""

		return
	}

	append description "$settling_time "

# Append the measurement type.

	set measurement_type \
	    [ mxgui_scan_get_measurement_type_from_gui $measurement_frame ]

	append description "$measurement_type "

# Append the measurement information.

	switch -exact -- $measurement_type {
		preset_time { set measurement_args \
	[ mxgui_scan_get_preset_time_args $measurement_frame.arguments_frame ]}

		preset_count { set measurement_args \
	[ mxgui_scan_get_preset_count_args $measurement_frame.arguments_frame ]}

		default { set measurement_args \
	[ mxgui_scan_get_default_args $measurement_frame.arguments_frame ]}
	}

	if { $measurement_args == "" } {
		return
	}

	append description "$measurement_args "

# Append the datafile information.

	set datafile_frame $window.datafile_plot_frame.datafile_frame

	append description "[ string tolower [ $datafile_frame.type get ] ] "

	append description "[ $datafile_frame.name get ] "

# Append the plot information.

	set plot_frame $window.datafile_plot_frame.plotfile_frame

	append description "[ string tolower [ $plot_frame.type get ] ] "

	append description "\"[ $plot_frame.arguments get ]\" "

# Append the scan range information.

	if { $num_motors <= 0 } {
		set num_steps [ $window.scan_steps_frame.num_scan_steps get ]

		append description "0 1 $num_steps"
	} else {
		list start_list
		list end_list
		list numsteps_list

		for { set i 0 } { $i < $num_motors } { incr i } {
			lappend start_list \
				[ $window.motor_frame.motor${i}_start get ]

			lappend end_list \
				[ $window.motor_frame.motor${i}_end get ]

			lappend numsteps_list \
				[ $window.motor_frame.motor${i}_numsteps get ]
		}
		append description "$start_list $end_list $numsteps_list"
	}

# Create the new scan record.

	if { $old_record_exists != 0 } {
		$scan_record delete_record
	}

	if [ catch { $record_list create_record_from_description \
					$description } new_record ] {

		set t "Attempt to create scan '$scan_name' failed."

		Mx::error_message "Record Creation Failed" $t "" ""

		return
	}

	if [ catch { $new_record finish_record_initialization } ] {

		set t "Attempt to initialize scan record '$scan_name' failed."

		Mx::error_message "Record Initialization Failed" $t "" ""

		return
	}

# Get rid of the scan setup window

	wm withdraw $window
	destroy $window

# Remember that we now have unsaved scans.

	$record_list configure -unsaved_scans 1
}

proc mxgui_scan_get_preset_time_args { arguments_frame } {

	set measurement_time [ $arguments_frame.measurement_time get ]

	set length [ string length [ string trim $measurement_time ] ]

	if { $length == 0 } {
		Mx::error_message "Empty measurement time" \
			"No measurement time was specified." "" ""

		return ""
	}

	set timer_name [ $arguments_frame.timer_name get ]

	return "\"$measurement_time $timer_name\""
}

proc mxgui_scan_get_preset_count_args { arguments_frame } {

	set measurement_counts [ $arguments_frame.measurement_counts get ]

	if { $length == 0 } {
		Mx::error_message "Empty measurement counts" \
			"No measurement counts was specified." "" ""

		return ""
	}

	set scaler_name [ $arguments_frame.scaler_name get ]

	return "\"$measurement_counts $scaler_name\""
}

proc mxgui_scan_get_default_args { arguments_frame } {

	set measurement_arguments [ $arguments_frame.measurement_arguments get ]

	return "\"$measurement_arguments\""
}

####################################################################
#                                                                  #
# The following routine calls mxgui_quick_create_scan to create a  #
# new scan record and then runs the scan.                          #
#                                                                  #
####################################################################

proc mxgui_quick_create_and_run_scan { window record_list \
				scan_name scan_record \
				scan_type old_record_exists } {

	mxgui_quick_create_scan $window $record_list $scan_name \
		$scan_record $scan_type $old_record_exists

	#
	#  Check to see if the files created by this scan will
	#  overwrite already existing files.
	#

	set status [ mxgui_check_for_datafile_name_collision $scan_record ]

	#
	#  If the returned status is 0, the user has decided to
	#  abort the scan since preexisting files would be
	#  overwritten.
	#

	if { $status == 0 } {
		return
	}

	#
	#  Prompt the user for scan header changes
	#

	set datafile_type [ get_datafile_type $scan_record ]

	set status [ menubar_setup_scan_header_change_dialog \
				$record_list $datafile_type $scan_record ]

	#
	#  If the returned status is 1, we have to do the scan ourself.
	#  Otherwise, a callback that was set up in the procedure
	#  menubar_setup_scan_header_change_dialog will do it for us.
	#

	if { $status == 1 } {
		mxgui_perform_scan $scan_record
	}
}

