#
#  Copyright 1999-2006 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

package provide MxGui 1.2

###########################################################################
#                                                                         #
#  The following routine is the main entry point for the scan setup and   #
#  modification screens.                                                  #
#                                                                         #
###########################################################################

proc mxgui_scan_setup { record_list scan_name scan_type } {

	set scan_record [ Mx::object_name $scan_name ]

# Does a record by this name already exist?

	set old_record_list [ itcl::find objects $scan_record ]

	set old_record_exists [ llength $old_record_list ]

# If the new scan name already exists, check to see if it is a scan record.

	if { $old_record_exists } {
		set isa_scan [ $scan_record isa Mx::Scan ]

		if { $isa_scan == 0 } {
			set t \
			  "A record named '$scan_name' already exists,\n"
			append t "but it is not a scan record, so it cannot\n"
			append t "be overwritten."

			Mx::error_message "Illegal Scan Name" $t "" ""
			return -error
		}

	}

# Dispatch scans the procedure for the requested scan type.

	switch -exact -- $scan_type {
		input_scan  -
		slit_scan   -
		pseudomotor_scan -
		motor_scan  { mxgui_motor_scan_setup $record_list \
			$scan_name $scan_type $scan_record $old_record_exists }

		mcs_qscan  { mxgui_quick_scan_setup $record_list \
			$scan_name $scan_type $scan_record $old_record_exists }

		quick_scan  { mxgui_quick_scan_setup $record_list \
			$scan_name mcs_qscan $scan_record $old_record_exists }

		2theta_scan { mxgui_2theta_scan_setup $record_list \
			$scan_name $scan_record $old_record_exists }

		file_list_scan { mxgui_list_scan_setup $record_list \
			$scan_name $scan_type $scan_record $old_record_exists }

		list_scan   { mxgui_list_scan_setup $record_list \
		    $scan_name file_list_scan $scan_record $old_record_exists }

		xafs_scan   { mxgui_xafs_scan_setup $record_list \
			$scan_name $scan_record $old_record_exists }

		default     { Mx::error_message "Unknown scan type" \
			"Scan type '$scan_type' is not recognized." "" ""

			    return }
	}
}

###########################################################################

proc mxgui_2theta_scan_setup { record_list scan_name \
				scan_record old_record_exists } {

	Mx::error_message "Unimplemented scan type" \
"Support within the MX GUI for scan type '2theta_scan' is not yet implemented."\
	"" ""
}

###########################################################################
#                                                                         #
#  The following procedure initializes the display of the number of times #
#  that the current scan will be performed when executed.                 #
#                                                                         #
###########################################################################

proc mxgui_initialize_num_scans_field { window record_list scan_record \
						old_record_exists } {

	set num_scans_frame $window.num_scans_frame

	frame $num_scans_frame
	pack $num_scans_frame -padx 5 -pady 5

	if { $old_record_exists == 0 } {
		set num_scans 1
	} else {
		set num_scans [ $scan_record get_field num_scans ]
	}

	iwidgets::entryfield $num_scans_frame.num_scans \
		-labeltext "Number of times to run this scan:" -width 8

	$num_scans_frame.num_scans clear
	$num_scans_frame.num_scans insert 0 $num_scans

	$num_scans_frame.num_scans configure -validate \
					{ iwidgets::Entryfield::integer %P }

	pack $num_scans_frame.num_scans -side top

}

###########################################################################
#                                                                         #
#  The following procedure is the routine normally used to display all    #
#  the fields below the scan range area on the GUI.                       #
#                                                                         #
###########################################################################

proc mxgui_initialize_standard_scan_display { window record_list scan_record \
					scan_type old_record_exists \
					show_measurement_parameters } {
#
# Put the input devices info and timer info in side by side frames.
#

	frame $window.readout_frame
	pack $window.readout_frame -padx 5 -pady 5
#
# First, initialize the input device display.
#
	set inputs_frame $window.readout_frame.inputs

	frame $inputs_frame

	mxgui_initialize_input_device_display $inputs_frame \
				$record_list $scan_record $old_record_exists

	frame $window.readout_frame.pad
	label $window.readout_frame.pad.text -text " " -width 4
	pack $window.readout_frame.pad.text

#
# Display the measurement info parameters.
#

	set measurement_frame $window.readout_frame.measurement

	frame $measurement_frame

	mxgui_scan_display_measurement_fields $measurement_frame $record_list \
					$scan_record $scan_type \
					$old_record_exists \
					$show_measurement_parameters

	grid $inputs_frame $window.readout_frame.pad $measurement_frame

	frame $window.separator3 -height 2 -borderwidth 1 -relief sunken
	pack $window.separator3 -expand yes -fill both

#
# Display datafile and plot type parameters.
#

	set datafile_plot_frame $window.datafile_plot_frame

	frame $datafile_plot_frame
	pack $datafile_plot_frame -padx 5 -pady 5

	mxgui_scan_display_datafile_and_plot_fields $datafile_plot_frame \
				$scan_record $scan_type $old_record_exists

	frame $window.separator4 -height 2 -borderwidth 1 -relief sunken
	pack $window.separator4 -expand yes -fill both

	return
}

###########################################################################
#                                                                         #
#  The following procedure sets up the initial display of the input       #
#  device fields.                                                         #
#                                                                         #
###########################################################################

proc mxgui_initialize_input_device_display { window \
			record_list scan_record old_record_exists } {

	set num_inputs_frame    $window.num_inputs_frame
	set input_device_frame  $window.input_device_frame

	frame $num_inputs_frame
	pack $num_inputs_frame

	if { $old_record_exists == 0 } {
		set num_input_devices 1
	} else {
		set num_input_devices \
			[ $scan_record get_field num_input_devices ]
	}

	iwidgets::entryfield $num_inputs_frame.num_input_devices \
			-labeltext "Number of input devices:" -width 8

	$num_inputs_frame.num_input_devices clear
	$num_inputs_frame.num_input_devices insert 0 $num_input_devices

	$num_inputs_frame.num_input_devices configure -validate \
					{ iwidgets::Entryfield::integer %P }

	pack $num_inputs_frame.num_input_devices -side left

#
# old_num_input_devices is meant to be a hidden field that stores the
# old number of input devices in a place the other procedures can find it.
# So don't make it visible by packing it or gridding it, etc.
#
	label $num_inputs_frame.old_num_input_devices -text 0

	frame $input_device_frame
	pack $input_device_frame

	label $input_device_frame.name -text "Input device name"

	grid $input_device_frame.name -column 1

	$num_inputs_frame.num_input_devices configure -command [ itcl::code \
		mxgui_scan_redo_input_device_fields \
			$input_device_frame $num_inputs_frame \
			$record_list $scan_record $old_record_exists ]

# Initialize the input device display.

	mxgui_scan_redo_input_device_fields \
			$input_device_frame $num_inputs_frame \
			$record_list $scan_record $old_record_exists

}

###########################################################################
#                                                                         #
#  The following routine causes the input device fields to be redisplayed #
#  after any change in the number of input devices for the scan.          #
#                                                                         #
###########################################################################

proc mxgui_scan_redo_input_device_fields { \
		input_device_frame num_inputs_frame \
		record_list scan_record old_record_exists } {

	set old_num_input_devices \
		[ $num_inputs_frame.old_num_input_devices cget -text ]

	set num_input_devices [ $num_inputs_frame.num_input_devices get ]

	if { $num_input_devices > $old_num_input_devices } {

		# Add new input devices to the list of input devices.

		for { set i $old_num_input_devices } \
				{ $i < $num_input_devices } { incr i } {

			set thisdevice $input_device_frame.device$i

			label ${thisdevice}_label -text "Device $i:"

			iwidgets::entryfield ${thisdevice}_name -command [ \
				itcl::code mxgui_scan_change_input_device \
				$record_list $thisdevice ]

			set arguments \$f\[$i\]

			label ${thisdevice}_plot -text $arguments

			grid ${thisdevice}_label ${thisdevice}_name \
				${thisdevice}_plot
		}
	} else {

		# Delete old input devices from the list.

		for { set i $num_input_devices } \
				{ $i < $old_num_input_devices } { incr i } {

			set thisdevice $input_device_frame.device$i

			destroy ${thisdevice}_label
			destroy ${thisdevice}_name
			destroy ${thisdevice}_plot
		}
	}

	$num_inputs_frame.old_num_input_devices configure \
				-text $num_input_devices
}

proc mxgui_scan_copy_input_device_fields_from_record { window \
						record_list scan_record } {

	set num_input_devices [ $scan_record get_field num_input_devices ]
	set input_device_list \
		[ split [ $scan_record get_field input_device_array ] ]

	set input_device_frame  $window.readout_frame.inputs.input_device_frame

	for { set i 0 } { $i < $num_input_devices } { incr i } {

		set thisdevice $input_device_frame.device$i

		${thisdevice}_name insert end [ lindex $input_device_list $i ]
	}
}

proc mxgui_scan_change_input_device { record_list thisdevice } {

	set device_name [ ${thisdevice}_name get ]

	if [ catch { $record_list get_record $device_name } device_record ] {
		Mx::error_message "No Such Device" \
		"There is no record named '$device_name'" "" ""

		${thisdevice}_name clear

		return
	}
	set superclass [ $device_record get_field mx_superclass ]

	switch -- $superclass {
		variable { return }
		device {}
		default {
			set t "Record '$device_name' is of superclass "
			append t "'$superclass' which is not allowed as "
			append t "an input device."

			Mx::error_message "Illegal Record Superclass" $t "" ""

			${thisdevice}_name clear

			return
		}
	}

	set class [ $device_record get_field mx_class ]

	switch -- $class {
		analog_input  -
		digital_input -
		motor         -
		scaler {}
		default {
			set t "Device '$device_name' may not be used as "
			append t "an input device.  Only scalers, motors, "
			append t "analog inputs, digital inputs, and "
			append t "variables may be used as input devices."

			Mx::error_message "Illegal Record Class" $t "" ""

			${thisdevice}_name clear

			return
		}
	}
}

###########################################################################
#                                                                         #
#      The following routine displays measurement parameter fields.       #
#                                                                         #
###########################################################################

proc mxgui_scan_display_measurement_fields { measurement_frame record_list \
					scan_record scan_type \
					old_record_exists \
					show_measurement_parameters } {

	if { $show_measurement_parameters != 0 } {
		iwidgets::optionmenu $measurement_frame.measurement_type \
			-labeltext "Measurement type:"

		$measurement_frame.measurement_type insert end \
				"Preset Time" "Preset Count"

		$measurement_frame.measurement_type configure -command \
			[ itcl::code mxgui_scan_display_measurement_type \
					$measurement_frame $record_list \
					$scan_record $scan_type \
					$old_record_exists "" ]

		frame $measurement_frame.arguments_frame

		pack $measurement_frame.measurement_type -anchor w -padx 2
		pack $measurement_frame.arguments_frame -anchor w -padx 2
	}

	set frame $measurement_frame.sframe

	frame $frame
	pack $frame -anchor w -padx 2

	iwidgets::entryfield $frame.settling_time \
				-labeltext "Settling time:" -width 10

	if { $old_record_exists == 0 } {
		$frame.settling_time insert end 0
	} else {
		$frame.settling_time insert end \
			[ $scan_record get_field "settling_time" ]
	}

	label $frame.settling_time_units -text "sec"

	pack $frame.settling_time -side left
	pack $frame.settling_time_units -side left

	if { $show_measurement_parameters != 0 } {
		if { $old_record_exists == 0 } {
			set measurement_type "preset_time"
		} else {
			set measurement_type \
				[ $scan_record get_field "measurement_type" ]
		}

		mxgui_scan_display_measurement_type $measurement_frame \
				$record_list $scan_record $scan_type \
				$old_record_exists $measurement_type
	}
}

proc mxgui_scan_get_measurement_type_from_gui { measurement_frame } {

	set measurement_type [ $measurement_frame.measurement_type get ]

	set measurement_type [ string tolower $measurement_type ]

	set measurement_list [ split $measurement_type " " ]

	set measurement_type [ join $measurement_list "_" ]

	return $measurement_type
}

proc mxgui_scan_display_measurement_type { measurement_frame record_list \
					scan_record scan_type \
					old_record_exists measurement_type } {

	if { $measurement_type == "" } {
		set measurement_type \
		[ mxgui_scan_get_measurement_type_from_gui $measurement_frame ]
	}

	set frame $measurement_frame.arguments_frame

	# Destroy whatever is already in the arguments frame.

	set child_list [ winfo children $frame ]

	foreach widget $child_list {
		destroy $widget
	}

	# Now rebuild the arguments frame depending on the measurement type.

	switch -exact -- $measurement_type {
		preset_time { mxgui_scan_display_preset_time \
	    $frame $record_list $scan_record $scan_type $old_record_exists }

		preset_count { mxgui_scan_display_preset_count \
	    $frame $record_list $scan_record $scan_type $old_record_exists }

		default { mxgui_scan_display_default_measurement \
	    $frame $record_list $scan_record $scan_type $old_record_exists }
	}
	return
}

proc mxgui_scan_display_preset_time { arguments_frame record_list \
				scan_record scan_type old_record_exists } {

	iwidgets::entryfield $arguments_frame.measurement_time \
		-labeltext "Measurement time:" -width 10

	label $arguments_frame.measurement_time_units -text "sec"

	iwidgets::optionmenu $arguments_frame.timer_name \
		-labeltext "Timer:"

	if { $scan_type == "mcs_qscan" } {
		set timer_list [ $record_list \
			get_multiple_records "device" "timer" "mcs_timer" 0 ]
	} else {
		set timer_list [ $record_list \
			get_multiple_records "device" "timer" ".*" 0 ]
	}

	if { [ llength $timer_list ] == 0 } {
		Mx::error_message "No Timers Available" \
		  "There are no timer records in the current database." "" ""

		return
	}

	foreach timer $timer_list {
		$arguments_frame.timer_name insert end $timer
	}

	if { $old_record_exists == 0 } {
		set measurement_time 0

		set current_timer ""
	} else {
		set old_measurement_type \
			[ $scan_record get_field "measurement_type" ]

		if { $old_measurement_type != "preset_time" } {
			set measurement_time 0

			set current_timer ""
		} else {
			set measurement_args \
			    [ $scan_record get_field "measurement_arguments" ]

			set measurement_args \
				[ string trim $measurement_args "\"" ]

			scan $measurement_args "%s %s" \
				measurement_time current_timer
		}
	}
	$arguments_frame.measurement_time insert end $measurement_time

	if { $current_timer != "" } {
		$arguments_frame.timer_name select $current_timer
	}

	pack $arguments_frame.measurement_time -side left
	pack $arguments_frame.measurement_time_units -side left
	pack $arguments_frame.timer_name -side left
}

proc mxgui_scan_display_preset_count { arguments_frame record_list \
				scan_record scan_type old_record_exists } {

	iwidgets::entryfield $arguments_frame.measurement_counts \
		-labeltext "Measurement counts:" -width 10

	label $arguments_frame.measurement_counts_units -text "counts"

	iwidgets::optionmenu $arguments_frame.scaler_name \
		-labeltext "Scaler:"

	if { $scan_type == "mcs_qscan" } {
		set scaler_list [ $record_list \
			get_multiple_records "device" "scaler" "mcs_scaler" 0 ]
	} else {
		set scaler_list [ $record_list \
			get_multiple_records "device" "scaler" ".*" 0 ]
	}

	if { [ llength $scaler_list ] == 0 } {
		Mx::error_message "No Scalers Available" \
		  "There are no scaler records in the current database." "" ""

		return
	}

	foreach scaler $scaler_list {
		$arguments_frame.scaler_name insert end $scaler
	}

	if { $old_record_exists == 0 } {
		set measurement_counts 0

		set current_scaler ""
	} else {
		set old_measurement_type \
			[ $scan_record get_field "measurement_type" ]

		if { $old_measurement_type != "preset_count" } {
			set measurement_counts 0

			set current_scaler ""
		} else {
			set measurement_args \
			    [ $scan_record get_field "measurement_arguments" ]

			set measurement_args \
				[ string trim $measurement_args "\"" ]

			scan $measurement_args "%s %s" \
				measurement_counts current_scaler
		}
	}
	$arguments_frame.measurement_counts insert end $measurement_counts

	if { $current_scaler != "" } {
		$arguments_frame.scaler_name select $current_scaler
	}

	pack $arguments_frame.measurement_counts -side left
	pack $arguments_frame.measurement_counts_units -side left
	pack $arguments_frame.scaler_name -side left
}

proc mxgui_scan_display_default_measurement { arguments_frame record_list \
				scan_record scan_type old_record_exists } {

	iwidgets::entryfield $arguments_frame.measurement_time \
		-labeltext "Measurement arguments:"

	pack $arguments_frame.measurement_arguments -side left
}

proc mxgui_scan_copy_measurement_fields_from_record { window record_list \
				scan_record show_measurement_parameters } {
	
	set measurement_frame $window.readout_frame.measurement

	if { $show_measurement_parameters != 0 } {
#		$measurement_frame.integration_time clear

#		$measurement_frame.integration_time insert 0 \
#			[ $scan_record get_field integration_time ]
	}

	$measurement_frame.sframe.settling_time clear

	$measurement_frame.sframe.settling_time insert end \
			[ $scan_record get_field settling_time ]
}

###########################################################################
#                                                                         #
#      The following routine displays datafile and plot fields.           #
#                                                                         #
###########################################################################

proc mxgui_scan_display_datafile_and_plot_fields { datafile_plot_frame \
				scan_record scan_type old_record_exists } {

	set datafile_frame $datafile_plot_frame.datafile_frame

	frame $datafile_frame
	pack $datafile_frame -side left 

	iwidgets::optionmenu $datafile_frame.type -labeltext "Datafile type:"

	$datafile_frame.type insert end SFF Text XAFS

	pack $datafile_frame.type

	iwidgets::entryfield $datafile_frame.name -labeltext "Datafile name:"

	pack $datafile_frame.name

	###################################################################

	label $datafile_plot_frame.pad -text " " -width 10
	pack $datafile_plot_frame.pad -side left -expand yes -fill x

	###################################################################

	set plotfile_frame $datafile_plot_frame.plotfile_frame

	frame $plotfile_frame
	pack $plotfile_frame -side left

	iwidgets::optionmenu $plotfile_frame.type -labeltext "Plotfile type:"

	$plotfile_frame.type insert end Gnuplot GnuXAFS None

	pack $plotfile_frame.type

	iwidgets::entryfield $plotfile_frame.arguments \
						-labeltext "Plot arguments:"

	pack $plotfile_frame.arguments

# Initialize the fields depending on the scan type.

	set datafile_name "junk.out"

	switch -- $scan_type {
		xafs_scan { 
			set datatype_selection XAFS
			set plotfile_selection GnuXAFS
			set plot_arguments "\$f\[0\]/\$f\[1\]"
		}
		default {
			set datatype_selection SFF
			set plotfile_selection Gnuplot
			set plot_arguments "\$f\[0\]"
		}
	}

	$datafile_frame.type select $datatype_selection

	$datafile_frame.name insert 0 $datafile_name

	$plotfile_frame.type select $plotfile_selection

	$plotfile_frame.arguments insert 0 $plot_arguments
}

proc mxgui_scan_copy_datafile_and_plot_fields_from_record { window \
						record_list scan_record } {

	set datafile_type [ get_datafile_type $scan_record ]
	set datafile_name [ $scan_record get_field datafile_name ]

	set thisframe $window.datafile_plot_frame

# Handle the datafile fields.

	$thisframe.datafile_frame.name clear
	$thisframe.datafile_frame.name insert 0 $datafile_name

	switch -- $datafile_type {
		text { set selection Text }
		sff  { set selection SFF }
		xafs { set selection XAFS }
		default { set selection Unknown }
	}

	$thisframe.datafile_frame.type select $selection

# Handle the plot fields

	set plot_type      [ get_plot_type $scan_record ]
	set plot_arguments [ $scan_record get_field plot_arguments ]

	set plot_arguments [ string trim $plot_arguments \" ]

	$thisframe.plotfile_frame.arguments clear
	$thisframe.plotfile_frame.arguments insert 0 $plot_arguments

	switch -- $plot_type {
		gnuplot { set selection Gnuplot }
		gnuxafs { set selection GnuXAFS }
		none    { set selection None }
		default { set selection Unknown }
	}

	$thisframe.plotfile_frame.type select $selection
}

###########################################################################
#                                                                         #
# mxgui_perform_scan is used to run a scan and modify the top level gui   #
# display as necessary.                                                   #
#                                                                         #
###########################################################################

proc mxgui_perform_scan { scan_record } {

	set scan_name [ $scan_record cget -name ]

	set window .scan_control_window

	frame $window
	pack $window -before .message_log

	label $window.scan_label -text "Current scan: "

	label $window.scan_name -text $scan_name -font $MxGui::monospaced_font

	pack $window.scan_label $window.scan_name -side left

	frame $window.sep 
	pack $window.sep -side left -expand yes

	label $window.sep.junk
	pack $window.sep.junk

	button $window.pause_scan -background orange -activebackground orange \
		-text "Pause Scan" -command [ itcl::code mxgui_pause_scan ]

	pack $window.pause_scan -side right -padx 5

	button $window.abort_scan -background red -activebackground red \
		-text "Abort Scan" -command [ itcl::code mxgui_abort_scan ]

	pack $window.abort_scan -side right -padx 5

	update

	catch { $scan_record perform_scan }

	update

	pack forget $window
	destroy $window

	update
}

proc mxgui_abort_scan {} {
	log_puts "Scan abort requested."

	set MxGui::user_interrupt 1
}

proc mxgui_pause_scan {} {
	set MxGui::user_interrupt 2
}

###########################################################################
#                                                                         #
# mxgui_check_for_datafile_name_collision checks to see if one or more of #
# the datafiles to be created by the specified scan record will overwrite #
# an already existing file.                                               #
#                                                                         #
###########################################################################

proc mxgui_check_for_datafile_name_collision { scan_record } {

	#
	# If the variable $MxGui::overwrite_enabled is non-zero, then
	# the overwriting of data files is always permitted.
	#

	if { $MxGui::overwrite_enabled != 0 } {
		return 1
	}

	# Split the scan's datafile field into its component parts.

	set datafile_type [ get_datafile_type $scan_record ]
	set datafile_name [ $scan_record get_field datafile_name ]

	if { $datafile_type == "none" } {
		return 1
	}

	# Find the version number, if any, in the datafile name.

	mxgui_parse_datafile_name $datafile_name \
		version_index version_number version_number_length

	set num_scans [ $scan_record get_field num_scans ]

	set num_existing_files 0

	if { $num_scans <= 0 } {
		set t "This scan has an illegal number of scans ($num_scans)"

		Mx::error_message "Bad Number of Scans" $t "" ""

		return 0

	} elseif { $num_scans == 1 } {

		# Single scan

		set status [ mxgui_test_datafile_name $datafile_name ]

		switch -- $status {
			readonly_file_exists { 
				if { $num_existing_files == 0 } {
					bell
log_puts "*** Warning ***"
				}
log_puts "  The output file '$datafile_name' already exists and is readonly."
log_puts "*** Scan aborted ***"
				return 0
			}
			file_exists {
				if { $num_existing_files == 0 } {
					bell
log_puts "*** Warning ***"
				}
				incr num_existing_files
log_puts "  The output file '$datafile_name' already exists."
			}
			file_does_not_exist  { }
			default {
				bell
log_puts "  Illegal status '$status' returned by mxgui_test_datafile_name"
log_puts "*** Scan aborted ***"
				return 0
			}
		}
	} else {
		# Multiple scans

		if { $version_index == -1 } {

			# Single filename

			set t \
"The output filename '$datafile_name' for this scan does not contain "
			append t \
"a version number.  Since this scan is to be run $num_scans times in a row, "
			append t \
"this means that the output of later passes will overwrite the output "
			append t \
"from earlier passes.\n\nProceed anyway?"

			set selection [ tk_dialog .overwrite_same_file_dialog \
"No Filename Version Number" $t question 1 "Yes" "No" ]

			if { $selection == 1 } {
log_puts "*** Scan aborted ***"
				return 0
			}

			set status [ mxgui_test_datafile_name $datafile_name ]

			switch -- $status {
				readonly_file_exists { 
					if { $num_existing_files == 0 } {
						bell
log_puts "*** Warning ***"
					}
log_puts "  The output file '$datafile_name' already exists and is readonly."
log_puts "*** Scan aborted ***"
					return 0
				}
				file_exists {
					if { $num_existing_files == 0 } {
						bell
log_puts "*** Warning ***"
					}
					incr num_existing_files
log_puts "  The output file '$datafile_name' already exists."
				}
				file_does_not_exist  { }
				default {
					bell
log_puts "  Illegal status '$status' returned by mxgui_test_datafile_name"
log_puts "*** Scan aborted ***"
					return 0
				}
			}
		} else {
			# Multiple filenames

			set namestem [ string range \
				$datafile_name 0 $version_number_length ]

			for { set i 0 } { $i < $num_scans } { incr i } {

				set new_version_number \
					[ expr ( $version_number + $i ) ]

				set new_version_string \
				    [ mxgui_construct_datafile_version_number \
					$new_version_number \
					$version_number_length ]

				set new_datafile_name [ format "%s.%s" \
					$namestem $new_version_string ]

				set status [ mxgui_test_datafile_name \
						$new_datafile_name ]

				switch -- $status {
					readonly_file_exists { 
					    if { $num_existing_files == 0 } {
						bell
log_puts "*** Warning ***"
					    }
log_puts "  The output file '$new_datafile_name' already exists and is readonly."
log_puts "*** Scan aborted ***"
					    return 0
					}
					file_exists {
					    if { $num_existing_files == 0 } {
						bell
log_puts "*** Warning ***"
					    }
					    incr num_existing_files
log_puts "  The output file '$new_datafile_name' already exists."
					}
					file_does_not_exist  { }
					default {
					    bell
log_puts "  Illegal status '$status' returned by mxgui_test_datafile_name"
log_puts "*** Scan aborted ***"
					    return 0
					}
				}
			}
		}
	}
	if { $num_existing_files < 0 } {
log_puts "*** Program error: num_existing_files = $num_existing_files ***"
		return 0

	} elseif { $num_existing_files == 0 } {
		return 1
	} elseif { $num_existing_files == 1 } {
		set t "There is an existing output file "
		append t "that would be overwritten by this scan.\n\n"
		append t "Overwrite it?"
	} else {
		set t "There are $num_existing_files existing output files "
		append t "that would be overwritten by this scan.\n\n"
		append t "Overwrite them?"
	}
	set selection [ tk_dialog .overwrite_files_dialog \
		"Overwrite Existing Files?" $t question 1 "Yes" "No" ]

	if { $selection == 1 } {
		log_puts "*** Scan aborted ***"
		return 0
	}
	return 1
}

proc mxgui_parse_datafile_name { datafile_name version_index_up \
				version_number_up version_number_length_up } {

	upvar $version_index_up         version_index
	upvar $version_number_up        version_number
	upvar $version_number_length_up version_number_length

	set extension_index [ string last "." $datafile_name ]

	if { $extension_index == -1 } {
		return
	}

	incr extension_index

	set extension_string \
		[ string range $datafile_name $extension_index end ]

	set extension_string_length [ string length $extension_string ]

	scan $extension_string "%d" extension_string_value

	if { [ info exists extension_string_value ] == 0 } {
		set extension_string_value -1

	} elseif { $extension_string_value < 0 } {
		set extension_string_value -1
	}

	if { $extension_string_value == -1 } {
		set version_index -1
		set version_number -1
		set version_number_length -1
	} else {
		set version_index $extension_index
		set version_number $extension_string_value
		set version_number_length $extension_string_length
	}
}

proc mxgui_test_datafile_name { datafile_name } {

	if [ file exists $datafile_name ] {
		if [ file writable $datafile_name ] {
			return "file_exists"
		} else {
			return "readonly_file_exists"
		}
	} else {
		return "file_does_not_exist"
	}
}

proc mxgui_construct_datafile_version_number { version_number \
					version_number_length } {

	set version_string \
		[ format "%0*d" $version_number_length $version_number ]

	# Only the last $version_number_length characters should be
	# present in the final version number string, so we strip off
	# any high order digits.

	set string_length [ string length $version_string ]

	set string_start [ expr ( $string_length - $version_number_length ) ]

	set version_string [ string range $version_string $string_start end ]

	return $version_string
}

